@@ -11,6 +11,9 @@
 #' If \code{rule.type = "gic"}, generalized information criterion rule is used,
 #' else if \code{rule.type = "cv"}, cross validation is used.
 #' Any unambiguous substring can be given. Default \code{rule.type = "gic"}.
+#' @param noise.var the variance of noise.
+#' @param init if init = FALSE(the default), the missing entries will initialize with mean.
+#' @param init.mat the initialization matrix.
 #' @param maxit.rank maximal number of iterations in searching rank. Default \code{maxit.rank = 1}.
 #' @param nfolds number of folds in cross validation. Default \code{nfolds = 5}.
 #' @param thresh convergence threshold, measured as the relative change in the Frobenius norm between two successive estimates.
@@ -40,7 +43,9 @@
 #' x_impute <- r.search(x_na, 1, 15, "rsvd", "gic")
 #' x_impute[["r.est"]]
 
-r.search <- function(x, r.min = 1, r.max, svd.method = c("tsvd", "rsvd"), rule.type = c("gic", "cv"),
+r.search <- function(x, r.min = 1, r.max = "auto",
+                     svd.method = c("tsvd", "rsvd"), rule.type = c("gic", "cv"), noise.var = 0,
+                     init = FALSE, init.mat = 0,
                      maxit.rank = 1, nfolds = 5, thresh = 1e-05, maxit = 100, override = FALSE, control = list(...), ...){
 
   m <- nrow(x)
@@ -53,6 +58,18 @@ r.search <- function(x, r.min = 1, r.max, svd.method = c("tsvd", "rsvd"), rule.t
   x_ob <- x_train[ind_ob]
   ind <- ind_ob - 1
 
+  ind_miss <- which(is.na(x_train), arr.ind = TRUE)
+
+  if(init == TRUE){
+    ind_miss <- cbind(ind_miss, as.vector(init.mat[ind_miss]))
+  }else{
+    ind_miss <- cbind(ind_miss, rnorm(nrow(ind_miss), 0, noise.var))
+  }
+  ind_miss[,1:2] <- ind_miss[,1:2] - 1
+  if(r.max == "auto"){
+    r.max = floor((m + n + sqrt((n + m)^2 - 4 * nrow(ind_ob)))/2)
+  }
+
   svdm <- match.arg(svd.method)
   type <- ifelse(svdm == "tsvd", 1, 2)
 
@@ -64,9 +81,9 @@ r.search <- function(x, r.min = 1, r.max, svd.method = c("tsvd", "rsvd"), rule.t
 
   if(rulet == "cv"){
     cvind <- sample(1 : length(x_ob), length(x_ob))
-    res <- cv_rank(ind[cvind,], x_ob[cvind], m, n, r.min, r.max, nfolds, maxit.rank, thresh, type)
+    res <- cv_rank(ind[cvind,], ind_miss, x_ob[cvind], m, n, r.min, r.max, nfolds, maxit.rank, thresh, type, init)
   }else{
-    Z_temp <- ic_rank(ind, x_ob, m, n, r.min, r.max, maxit.rank, thresh, type)
+    Z_temp <- ic_rank(ind, ind_miss, x_ob, m, n, r.min, r.max, maxit.rank, thresh, type, init)
     m1 <- max(m, n)
     m2 <- min(m, n)
     res <- m2*log(Z_temp) + rank.seq*log(log(m2)) * log(m1)
@@ -74,7 +91,7 @@ r.search <- function(x, r.min = 1, r.max, svd.method = c("tsvd", "rsvd"), rule.t
 
   r.est <- rank.seq[which.min(res)]
 
-  Z_temp <- kkt_fix(ind, x_ob, m, n, r.est, maxit, thresh, type)
+  Z_temp <- kkt_fix(ind, ind_miss, x_ob, m, n, r.est, maxit, thresh, type, init)
   Z.fit <- Z_temp[[1]] * (x_sd[[4]] %*% t(x_sd[[5]])) + matrix(rep(x_sd[[2]], n), nrow = m) + t(matrix(rep(x_sd[[3]], m), nrow = n))
   if (!override) {
     Z.fit[ind_ob] <- x_ob
